<?php

namespace Bata\Address\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bata\Address\Http\Requests\BataRequest;
use Bata\Address\Interfaces\BataRepositoryInterface;
use Bata\Address\Models\Bata;

/**
 * Resource controller class for bata.
 */
class BataResourceController extends BaseController
{

    /**
     * Initialize bata resource controller.
     *
     * @param type BataRepositoryInterface $bata
     *
     * @return null
     */
    public function __construct(BataRepositoryInterface $bata)
    {
        parent::__construct();
        $this->repository = $bata;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bata\Address\Repositories\Criteria\BataResourceCriteria::class);
    }

    /**
     * Display a list of bata.
     *
     * @return Response
     */
    public function index(BataRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bata\Address\Repositories\Presenter\BataPresenter::class)
                ->$function();
        }

        $batas = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('address::bata.names'))
            ->view('address::bata.index', true)
            ->data(compact('batas', 'view'))
            ->output();
    }

    /**
     * Display bata.
     *
     * @param Request $request
     * @param Model   $bata
     *
     * @return Response
     */
    public function show(BataRequest $request, Bata $bata)
    {

        if ($bata->exists) {
            $view = 'address::bata.show';
        } else {
            $view = 'address::bata.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('address::bata.name'))
            ->data(compact('bata'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new bata.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(BataRequest $request)
    {

        $bata = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('address::bata.name')) 
            ->view('address::bata.create', true) 
            ->data(compact('bata'))
            ->output();
    }

    /**
     * Create new bata.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(BataRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $bata                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('address::bata.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('address/bata/' . $bata->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/address/bata'))
                ->redirect();
        }

    }

    /**
     * Show bata for editing.
     *
     * @param Request $request
     * @param Model   $bata
     *
     * @return Response
     */
    public function edit(BataRequest $request, Bata $bata)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('address::bata.name'))
            ->view('address::bata.edit', true)
            ->data(compact('bata'))
            ->output();
    }

    /**
     * Update the bata.
     *
     * @param Request $request
     * @param Model   $bata
     *
     * @return Response
     */
    public function update(BataRequest $request, Bata $bata)
    {
        try {
            $attributes = $request->all();

            $bata->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('address::bata.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('address/bata/' . $bata->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('address/bata/' . $bata->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the bata.
     *
     * @param Model   $bata
     *
     * @return Response
     */
    public function destroy(BataRequest $request, Bata $bata)
    {
        try {

            $bata->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('address::bata.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('address/bata/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('address/bata/' . $bata->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple bata.
     *
     * @param Model   $bata
     *
     * @return Response
     */
    public function delete(BataRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('address::bata.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('address/bata'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/address/bata'))
                ->redirect();
        }

    }

    /**
     * Restore deleted batas.
     *
     * @param Model   $bata
     *
     * @return Response
     */
    public function restore(BataRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('address::bata.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/address/bata'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/address/bata/'))
                ->redirect();
        }

    }

}
